#!/usr/local/bin/python

from OpenGL.GL import *
from OpenGL.GLU import *
import random
from math import * # trigonometry

import pygame, sys,os, time, random
from pygame.locals import * 

import Image

from GLTools import *
from shaders import *

WINDOW_SIZE = [512,512,False]
WINDOW_SIZE = [800,600,False]
#WINDOW_SIZE = [1650,1050,True]

render_lines = False
rotate_cubes = False
use_blending = True
use_culling = False
use_postfx = True
t=100
dist = 0.5
voxelsize = 2
cwt = 0.8

#exit clean
def process_input(events):
    global render_lines,rotate_cubes,use_blending,voxelsize,use_culling,use_postfx,cwt
    
    for event in events: 
        if event.type == QUIT:
            return False
        if event.type == pygame.KEYDOWN:
            if event.key == pygame.K_ESCAPE: 
                return False
            if event.key == pygame.K_5:
                render_lines = not render_lines
            if event.key == pygame.K_2:
                rotate_cubes = not rotate_cubes
            if event.key == pygame.K_3:
                use_blending = not use_blending
            if event.key == pygame.K_4:
                use_culling = not use_culling
            if event.key == pygame.K_1:
                use_postfx = not use_postfx
            if event.key == pygame.K_KP_PLUS:
                voxelsize+=1
            if event.key == pygame.K_KP_MINUS:
                voxelsize-=1
            if event.key == pygame.K_KP_MULTIPLY:
                cwt += 0.05
            if event.key == pygame.K_KP_DIVIDE:
                cwt -= 0.05
        else: 
            pass #print event
    return True

# get an OpenGL surface
pygame.init()
flags = pygame.OPENGL|pygame.DOUBLEBUF|pygame.NOFRAME
if WINDOW_SIZE[2]:
    flags |= pygame.FULLSCREEN
pygame.display.set_mode((WINDOW_SIZE[0],WINDOW_SIZE[1]),flags) #pygame.FULLSCREEN|pygame.NOFRAME

# load a cube into a display list

box = Primitive("cube")
shader = Shader(light_vs, light_ps)
postFX = Shader(basic_vs, postfx_ps)
rt = RenderTexture( WINDOW_SIZE[0],WINDOW_SIZE[1],GL_RGB,{'wrap':GL_CLAMP} )

texture = Texture()
texture.load("data/tmt-logo.png")
last_time = pygame.time.get_ticks()
elapsed_time = 1000

conway = Shader(basic_vs, conway_ps)
conway_size = WINDOW_SIZE
conway_rts = [
    RenderTexture(conway_size[0],conway_size[1],GL_RGB,{'mipmaps':False,'wrap':GL_REPEAT,'mag_filter':GL_NEAREST,'min_filter':GL_NEAREST}),
    RenderTexture(conway_size[0],conway_size[1],GL_RGB,{'mipmaps':False,'wrap':GL_REPEAT,'mag_filter':GL_NEAREST,'min_filter':GL_NEAREST}) ]
eden_texture = Texture({'mipmaps':False,'wrap':GL_REPEAT,'min_filter':GL_NEAREST})
eden_texture.load("data/conway-eden.png")
conway_rts[1].enable()
eden_texture.render()
conway_rts[1].disable()

def conway_frame():
    conway.enable()
    conway_rts[0].enable() #front buffer
    conway.setUniformTexture("texture", conway_rts[1].texture_id)
    conway.setUniform2f("delta_cell", [1.0/conway_rts[0].width, 1.0/conway_rts[0].height])
    conway.setUniform1f("cwt", cwt)
    conway_rts[1].render() #quad
    if key[pygame.K_LSHIFT]:
        rt.render()
    conway_rts[0].disable()
    conway_rts[0],conway_rts[1] = conway_rts[1],conway_rts[0] #swap
    conway.disable()


while True:

    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(90,1,0.01,1000)
    gluLookAt(dist*sin(t/260.0)*4,dist*cos(t/260.0)*4,dist*cos(t/687.0)*3,0,0,0,0,1,0)

    if (use_postfx):
        rt.enable()

    glClearColor(0.05, 0.06, 0.07, 1.0)
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)

    glMatrixMode(GL_MODELVIEW)

    # calculate light source position
    ld=[sin(t/16.0)*4.0,sin(t/20.0)*4.0,cos(t/16.0)*4.0]
    # pass data to fragment shader
    glLightfv(GL_LIGHT0,GL_POSITION,[ld[0],ld[1],ld[2]]);

    # fallback
    glColor3f(1,1,1)

    glLoadIdentity()
    # render a pretty range of cubes

    if(use_blending):
        glBlendFunc( GL_ONE, GL_ONE )
        glEnable( GL_BLEND )
        glDisable(GL_DEPTH_TEST)
        glFrontFace(GL_CW)        
    else:
        glEnable(GL_DEPTH_TEST)
        glEnable(GL_CULL_FACE)
        glFrontFace(GL_CCW)

    if (use_culling):
        glEnable(GL_CULL_FACE)
    else:
        glDisable(GL_CULL_FACE)
        
    glLineWidth(1)

    pygame.event.pump()
    key=pygame.key.get_pressed()

    shader.enable()
    shader.setUniform1f("f",1.0)

    glColor(0.05,0.06,0.07)

    for i in range(-voxelsize,voxelsize):
        for j in range(-voxelsize,voxelsize):
            for k in range(-voxelsize,voxelsize):

                glPushMatrix()

                glTranslate(i,j,k)
                if (rotate_cubes):
                    glRotate(t*i*0.1,0,1,0)
                    glRotate(t*j*0.1,1,0,0)
                    
                f = 0.5 # + 0.1 * abs( sin(i + j + k) )
                f = 1 / max(1.0, distance([i,j,k],ld))
                f= max(f*f,0.2)
                
                glScale(f,f,f)
                box.render()

                if render_lines:
                    glScale(1.1,1.1,1.1)
                    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)
                    box.render()
                    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)
                glPopMatrix()

    #final cube
    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)
    glPushMatrix()
    glScale(voxelsize-1,voxelsize-1,voxelsize-1)
    box.render()
    glPopMatrix()
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)
                
    glDisable(GL_BLEND)
    shader.disable()    

    #TMT-logo
    glEnable( GL_BLEND )
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
    texture.render(0.75,0.26,0.2,-0.26)
    

    if (use_postfx):
        rt.disable()
        postFX.enable()
        postFX.setUniform1f("off",random.random()-0.5)
        postFX.setUniformTexture("texture",rt.texture_id)
        rt.render()
        postFX.disable()

    if key[pygame.K_SPACE]:
        dist *= 1.01
    if key[pygame.K_BACKSPACE]:
        dist *= 0.99

    if key[pygame.K_LCTRL]:
        conway_frame()
        conway_rts[1].render()
        #conway_rts[1].render(0.1,0.1,0.5,0.5)
   
    renderFPSbar(elapsed_time)
                        
    pygame.display.flip()
    if not process_input(pygame.event.get()):
        pygame.display.quit()
        sys.exit(0)

    current_time = pygame.time.get_ticks()
    elapsed_time = max(current_time - last_time,0.001)
    last_time = current_time

    t += elapsed_time * 0.03



