

light_vs="""
varying vec3 v;
varying vec3 N;
varying vec4 C;

void main(void)
{

v = (gl_ModelViewMatrix * gl_Vertex).xyz;
N = (gl_NormalMatrix * gl_Normal).xyz;
C = gl_Color;

gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;

}
""";

light_ps="""
varying vec3 N;
varying vec3 v;
varying vec4 C;
uniform float f;

void main(void)
{
vec3 L = gl_LightSource[0].position.xyz-v;

// "Lambert's law"? (see notes)
// Rather: faces will appear dimmer when struck in an acute angle
// distance attenuation

float Idiff = max(dot(normalize(L),N),0.0)*pow(length(L),-2.0);

gl_FragColor = C + f * vec4(1.0,1.0,1.0,1.0)*Idiff; // diffuse reflection
}
""";

basic_vs="""
void main(void)
{
    gl_TexCoord[0] = gl_MultiTexCoord0;
    gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;
}
"""

postfx_ps="""
uniform sampler2D texture;
uniform float off;

void main(void)
{
    vec4 final = vec4(0,0,0,1);
    float d = length(gl_TexCoord[0].st - vec2(0.5,0.5)) + off;
    final.x = texture2D(texture,gl_TexCoord[0].st - vec2(0.02 * d,0),d * 3).x;
    final.y = texture2D(texture,gl_TexCoord[0].st).y;
    final.z = texture2D(texture,gl_TexCoord[0].st + vec2(0.02 * d,0),d * 3).z;
    final *= abs( sin( gl_TexCoord[0].t * 600 ) );
    final += 0.5 * texture2D(texture,gl_TexCoord[0].st,3.0);
    
    gl_FragColor = final;
}
"""

conway_ps="""
uniform sampler2D texture;
uniform vec2 delta_cell;
uniform float cwt; //1.0 is a good value

int readCell(float x, float y)
{
    vec4 pixel = texture2D(texture,gl_TexCoord[0].st + vec2(x*delta_cell.x,y*delta_cell.y));
    if (pixel.x > 0.5)
        return 1;
    return 0;
}

void main(void)
{
    int alive = 0;
    int result = 0;
    
    alive += readCell(-1,+1);
    alive += readCell(+0,+1);
    alive += readCell(+1,+1);
    
    alive += readCell(-1,0);
    alive += readCell(+1,0);

    alive += readCell(-1,-1);
    alive += readCell(+0,-1);
    alive += readCell(+1,-1);

    int me = readCell(0,0);

    if (me == 0)
    {
        if (alive == 3)
            result = 1;
    }
    else //alive
    {
        if (alive == 2 || alive == 3)
            result = 1;
        else
            result = 0;
    }
    
    //vec4 pixel = (result ? vec4(1.0, 1.0, 1.0, 1.0) : vec4(0.0, 0.0, 0.0, 1.0));
    vec4 pixel = texture2D(texture,gl_TexCoord[0].st);
    pixel.z = pixel.y;
    pixel.y = pixel.x;
    //pixel.x = (result ? 1.0 : 0.0 );
    pixel.x += (result ? cwt : -cwt );
    gl_FragColor = pixel;
}
"""
